@@ -3,7 +3,7 @@
 from sqlalchemy import *
 from sqlalchemy.util import OrderedDict
 
-metadata = BoundMetaData('sqlite:///', echo=True)
+metadata = MetaData('sqlite:///', echo=True)
 
 trees = Table('treenodes', metadata,
     Column('node_id', Integer, Sequence('treenode_id_seq',optional=False), primary_key=True),
