@@ -15,12 +15,6 @@ import sqlalchemy.ansisql as ansisql
 import sqlalchemy.types as sqltypes
 import sqlalchemy.exceptions as exceptions
 
-try:
-    import kinterbasdb
-except:
-    kinterbasdb = None
-
-dbmodule = kinterbasdb
 
 _initialized_kb = False
 
@@ -33,7 +27,6 @@ class FBNumeric(sqltypes.Numeric):
             return "NUMERIC(%(precision)s, %(length)s)" % { 'precision': self.precision,
                                                             'length' : self.length }
 
-
 class FBInteger(sqltypes.Integer):
     def get_col_spec(self):
         return "INTEGER"
@@ -111,63 +104,60 @@ class FBExecutionContext(default.DefaultExecutionContext):
 
 
 class FBDialect(ansisql.ANSIDialect):
-    def __init__(self, module = None, **params):
-        global _initialized_kb
-        self.module = module or dbmodule
-        self.opts = {}
+    def __init__(self, type_conv=200, concurrency_level=1, **kwargs):
+        ansisql.ANSIDialect.__init__(self, **kwargs)
 
-        if not _initialized_kb:
-            _initialized_kb = True
-            type_conv = params.get('type_conv', 200) or 200
-            if isinstance(type_conv, types.StringTypes):
-                type_conv = int(type_conv)
-
-            concurrency_level = params.get('concurrency_level', 1) or 1
-            if isinstance(concurrency_level, types.StringTypes):
-                concurrency_level = int(concurrency_level)
-
-            if kinterbasdb is not None:
-                kinterbasdb.init(type_conv=type_conv, concurrency_level=concurrency_level)
-        ansisql.ANSIDialect.__init__(self, **params)
+        self.type_conv = type_conv
+        self.concurrency_level= concurrency_level
 
+    def dbapi(cls):
+        import kinterbasdb
+        return kinterbasdb
+    dbapi = classmethod(dbapi)
+    
     def create_connect_args(self, url):
         opts = url.translate_connect_args(['host', 'database', 'user', 'password', 'port'])
         if opts.get('port'):
             opts['host'] = "%s/%s" % (opts['host'], opts['port'])
             del opts['port']
         opts.update(url.query)
-        # pop arguments that we took at the module level
-        opts.pop('type_conv', None)
-        opts.pop('concurrency_level', None)
-        self.opts = opts
 
-        return ([], self.opts)
+        type_conv = opts.pop('type_conv', self.type_conv)
+        concurrency_level = opts.pop('concurrency_level', self.concurrency_level)
+        global _initialized_kb
+        if not _initialized_kb and self.dbapi is not None:
+            _initialized_kb = True
+            self.dbapi.init(type_conv=type_conv, concurrency_level=concurrency_level)
+        return ([], opts)
 
-    def create_execution_context(self):
-        return FBExecutionContext(self)
+    def create_execution_context(self, *args, **kwargs):
+        return FBExecutionContext(self, *args, **kwargs)
 
     def type_descriptor(self, typeobj):
         return sqltypes.adapt_type(typeobj, colspecs)
 
     def supports_sane_rowcount(self):
-        return True
+        return False
 
     def compiler(self, statement, bindparams, **kwargs):
         return FBCompiler(self, statement, bindparams, **kwargs)
 
     def schemagenerator(self, *args, **kwargs):
-        return FBSchemaGenerator(*args, **kwargs)
+        return FBSchemaGenerator(self, *args, **kwargs)
 
     def schemadropper(self, *args, **kwargs):
-        return FBSchemaDropper(*args, **kwargs)
+        return FBSchemaDropper(self, *args, **kwargs)
 
-    def defaultrunner(self, engine, proxy):
-        return FBDefaultRunner(engine, proxy)
+    def defaultrunner(self, connection):
+        return FBDefaultRunner(connection)
 
     def preparer(self):
         return FBIdentifierPreparer(self)
 
-    def has_table(self, connection, table_name):
+    def max_identifier_length(self):
+        return 31
+
+    def has_table(self, connection, table_name, schema=None):
         tblqry = """
         SELECT count(*)
         FROM RDB$RELATIONS R
@@ -180,6 +170,12 @@ class FBDialect(ansisql.ANSIDialect):
         else:
             return False
 
+    def is_disconnect(self, e):
+        if isinstance(e, self.dbapi.OperationalError):
+            return 'Unable to complete network request to host' in str(e)
+        else:
+            return False
+
     def reflecttable(self, connection, table):
         #TODO: map these better
         column_func = {
@@ -292,9 +288,6 @@ class FBDialect(ansisql.ANSIDialect):
         for name,value in fks.iteritems():
             table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1], name=name))
 
-    def last_inserted_ids(self):
-        return self.context.last_inserted_ids
-
     def do_execute(self, cursor, statement, parameters, **kwargs):
         cursor.execute(statement, parameters or [])
 
@@ -304,15 +297,6 @@ class FBDialect(ansisql.ANSIDialect):
     def do_commit(self, connection):
         connection.commit(True)
 
-    def connection(self):
-        """Returns a managed DBAPI connection from this SQLEngine's connection pool."""
-        c = self._pool.connect()
-        c.supportsTransactions = 0
-        return c
-
-    def dbapi(self):
-        return self.module
-
 
 class FBCompiler(ansisql.ANSICompiler):
     """Firebird specific idiosincrasies"""
@@ -328,19 +312,17 @@ class FBCompiler(ansisql.ANSICompiler):
         else:
             self.strings[func] = func.name
 
-    def visit_insert(self, insert):
-        """Inserts are required to have the primary keys be explicitly present.
-         mapper will by default not put them in the insert statement to comply
-         with autoincrement fields that require they not be present. So,
-         put them all in for all primary key columns."""
-        for c in insert.table.primary_key:
-            if not self.parameters.has_key(c.key):
-                self.parameters[c.key] = None
-        return ansisql.ANSICompiler.visit_insert(self, insert)
+    def visit_insert_column(self, column, parameters):
+        # all column primary key inserts must be explicitly present
+        if column.primary_key:
+            parameters[column.key] = None
 
     def visit_select_precolumns(self, select):
-        """Called when building a SELECT statement, position is just before column list
-        Firebird puts the limit and offset right after the select..."""
+        """Called when building a ``SELECT`` statement, position is just
+        before column list Firebird puts the limit and offset right
+        after the ``SELECT``...
+        """
+
         result = ""
         if select.limit:
             result += " FIRST %d "  % select.limit
@@ -351,14 +333,14 @@ class FBCompiler(ansisql.ANSICompiler):
         return result
 
     def limit_clause(self, select):
-        """Already taken care of in the visit_select_precolumns method."""
+        """Already taken care of in the `visit_select_precolumns` method."""
         return ""
 
 
 class FBSchemaGenerator(ansisql.ANSISchemaGenerator):
     def get_column_specification(self, column, **kwargs):
         colspec = self.preparer.format_column(column)
-        colspec += " " + column.type.engine_impl(self.engine).get_col_spec()
+        colspec += " " + column.type.dialect_impl(self.dialect).get_col_spec()
 
         default = self.get_column_default_string(column)
         if default is not None:
@@ -382,11 +364,11 @@ class FBSchemaDropper(ansisql.ANSISchemaDropper):
 
 class FBDefaultRunner(ansisql.ANSIDefaultRunner):
     def exec_default_sql(self, default):
-        c = sql.select([default.arg], from_obj=["rdb$database"], engine=self.engine).compile()
-        return self.proxy(str(c), c.get_params()).fetchone()[0]
+        c = sql.select([default.arg], from_obj=["rdb$database"]).compile(engine=self.connection)
+        return self.connection.execute_compiled(c).scalar()
 
     def visit_sequence(self, seq):
-        return self.proxy("SELECT gen_id(" + seq.name + ", 1) FROM rdb$database").fetchone()[0]
+        return self.connection.execute_text("SELECT gen_id(" + seq.name + ", 1) FROM rdb$database").scalar()
 
 
 RESERVED_WORDS = util.Set(
