@@ -5,66 +5,111 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 
-import sys, StringIO, string, re
+import sys, StringIO, string, re, warnings
 
-import sqlalchemy.util as util
-import sqlalchemy.sql as sql
-import sqlalchemy.engine as engine
-import sqlalchemy.engine.default as default
-import sqlalchemy.schema as schema
-import sqlalchemy.ansisql as ansisql
+from sqlalchemy import util, sql, engine, schema, ansisql, exceptions, logging
+from sqlalchemy.engine import default, base
 import sqlalchemy.types as sqltypes
-import sqlalchemy.exceptions as exceptions
 
-try:
-    import cx_Oracle
-except:
-    cx_Oracle = None
-        
+import datetime
+
+
 class OracleNumeric(sqltypes.Numeric):
     def get_col_spec(self):
-        return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+        if self.precision is None:
+            return "NUMERIC"
+        else:
+            return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+
 class OracleInteger(sqltypes.Integer):
     def get_col_spec(self):
         return "INTEGER"
+
 class OracleSmallInteger(sqltypes.Smallinteger):
     def get_col_spec(self):
         return "SMALLINT"
+
+class OracleDate(sqltypes.Date):
+    def get_col_spec(self):
+        return "DATE"
+    def convert_bind_param(self, value, dialect):
+        return value
+    def convert_result_value(self, value, dialect):
+        if not isinstance(value, datetime.datetime):
+            return value
+        else:
+            return value.date()
+
 class OracleDateTime(sqltypes.DateTime):
     def get_col_spec(self):
         return "DATE"
+        
+    def convert_result_value(self, value, dialect):
+        if value is None or isinstance(value,datetime.datetime):
+            return value
+        else:
+            # convert cx_oracle datetime object returned pre-python 2.4
+            return datetime.datetime(value.year,value.month,
+                value.day,value.hour, value.minute, value.second)
+
 # Note:
 # Oracle DATE == DATETIME
 # Oracle does not allow milliseconds in DATE
 # Oracle does not support TIME columns
 
 # only if cx_oracle contains TIMESTAMP
-class OracleTimestamp(sqltypes.DateTime):
+class OracleTimestamp(sqltypes.TIMESTAMP):
     def get_col_spec(self):
         return "TIMESTAMP"
+
     def get_dbapi_type(self, dialect):
         return dialect.TIMESTAMP
-        
-class OracleText(sqltypes.TEXT):
-    def get_col_spec(self):
-        return "CLOB"
+
+    def convert_result_value(self, value, dialect):
+        if value is None or isinstance(value,datetime.datetime):
+            return value
+        else:
+            # convert cx_oracle datetime object returned pre-python 2.4
+            return datetime.datetime(value.year,value.month,
+                value.day,value.hour, value.minute, value.second)
+
+
 class OracleString(sqltypes.String):
     def get_col_spec(self):
         return "VARCHAR(%(length)s)" % {'length' : self.length}
+
+class OracleText(sqltypes.TEXT):
+    def get_dbapi_type(self, dbapi):
+        return dbapi.CLOB
+
+    def get_col_spec(self):
+        return "CLOB"
+
+    def convert_result_value(self, value, dialect):
+        if value is None:
+            return None
+        else:
+            return super(OracleText, self).convert_result_value(value.read(), dialect)
+
+
 class OracleRaw(sqltypes.Binary):
     def get_col_spec(self):
         return "RAW(%(length)s)" % {'length' : self.length}
+
 class OracleChar(sqltypes.CHAR):
     def get_col_spec(self):
         return "CHAR(%(length)s)" % {'length' : self.length}
+
 class OracleBinary(sqltypes.Binary):
     def get_dbapi_type(self, dbapi):
-        return dbapi.BINARY
+        return dbapi.BLOB
+
     def get_col_spec(self):
         return "BLOB"
+
     def convert_bind_param(self, value, dialect):
-        # this is RAWTOHEX
-        return ''.join(["%.2X" % ord(c) for c in value])
+        return value
+
     def convert_result_value(self, value, dialect):
         if value is None:
             return None
@@ -74,10 +119,12 @@ class OracleBinary(sqltypes.Binary):
 class OracleBoolean(sqltypes.Boolean):
     def get_col_spec(self):
         return "SMALLINT"
+
     def convert_result_value(self, value, dialect):
         if value is None:
             return None
         return value and True or False
+
     def convert_bind_param(self, value, dialect):
         if value is True:
             return 1
@@ -86,16 +133,15 @@ class OracleBoolean(sqltypes.Boolean):
         elif value is None:
             return None
         else:
-            return value and True or False 	
+            return value and True or False
 
-        
 colspecs = {
     sqltypes.Integer : OracleInteger,
     sqltypes.Smallinteger : OracleSmallInteger,
     sqltypes.Numeric : OracleNumeric,
     sqltypes.Float : OracleNumeric,
     sqltypes.DateTime : OracleDateTime,
-    sqltypes.Date : OracleDateTime,
+    sqltypes.Date : OracleDate,
     sqltypes.String : OracleString,
     sqltypes.Binary : OracleBinary,
     sqltypes.Boolean : OracleBoolean,
@@ -106,7 +152,7 @@ colspecs = {
 
 ischema_names = {
     'VARCHAR2' : OracleString,
-    'DATE' : OracleDateTime,
+    'DATE' : OracleDate,
     'DATETIME' : OracleDateTime,
     'NUMBER' : OracleNumeric,
     'BLOB' : OracleBinary,
@@ -115,29 +161,9 @@ ischema_names = {
     'RAW' : OracleRaw,
     'FLOAT' : OracleNumeric,
     'DOUBLE PRECISION' : OracleNumeric,
+    'LONG' : OracleText,
 }
 
-constraintSQL = """SELECT
-  ac.constraint_name,
-  ac.constraint_type,
-  LOWER(loc.column_name) AS local_column,
-  LOWER(rem.table_name) AS remote_table,
-  LOWER(rem.column_name) AS remote_column,
-  LOWER(rem.owner) AS remote_owner
-FROM all_constraints ac,
-  all_cons_columns loc,
-  all_cons_columns rem
-WHERE ac.table_name = :table_name
-AND ac.constraint_type IN ('R','P')
-AND ac.owner = :owner
-AND ac.owner = loc.owner
-AND ac.constraint_name = loc.constraint_name
-AND ac.r_owner = rem.owner(+)
-AND ac.r_constraint_name = rem.constraint_name(+)
--- order multiple primary keys correctly
-ORDER BY ac.constraint_name, loc.position, rem.position"""
-
-
 def descriptor():
     return {'name':'oracle',
     'description':'Oracle',
@@ -148,26 +174,50 @@ def descriptor():
     ]}
 
 class OracleExecutionContext(default.DefaultExecutionContext):
-    def pre_exec(self, engine, proxy, compiled, parameters):
-        super(OracleExecutionContext, self).pre_exec(engine, proxy, compiled, parameters)
+    def pre_exec(self):
+        super(OracleExecutionContext, self).pre_exec()
         if self.dialect.auto_setinputsizes:
-                self.set_input_sizes(proxy(), parameters)
+            self.set_input_sizes()
+
+    def get_result_proxy(self):
+        if self.cursor.description is not None:
+            if self.dialect.auto_convert_lobs and self.typemap is None:
+                typemap = {}
+                binary = False
+                for column in self.cursor.description:
+                    type_code = column[1]
+                    if type_code in self.dialect.ORACLE_BINARY_TYPES:
+                        binary = True
+                        typemap[column[0].lower()] = OracleBinary()
+                self.typemap = typemap
+                if binary:
+                    return base.BufferedColumnResultProxy(self)
+            else:
+                for column in self.cursor.description:
+                    type_code = column[1]
+                    if type_code in self.dialect.ORACLE_BINARY_TYPES:
+                        return base.BufferedColumnResultProxy(self)
         
+        return base.ResultProxy(self)
+
 class OracleDialect(ansisql.ANSIDialect):
-    def __init__(self, use_ansi=True, auto_setinputsizes=False, module=None, threaded=True, **kwargs):
+    def __init__(self, use_ansi=True, auto_setinputsizes=True, auto_convert_lobs=True, threaded=True, **kwargs):
+        ansisql.ANSIDialect.__init__(self, default_paramstyle='named', **kwargs)
         self.use_ansi = use_ansi
         self.threaded = threaded
-        if module is None:
-            self.module = cx_Oracle
-        else:
-            self.module = module
-        self.supports_timestamp = hasattr(self.module, 'TIMESTAMP' )
+        self.supports_timestamp = self.dbapi is None or hasattr(self.dbapi, 'TIMESTAMP' )
         self.auto_setinputsizes = auto_setinputsizes
-        ansisql.ANSIDialect.__init__(self, **kwargs)
-
-    def dbapi(self):
-        return self.module
+        self.auto_convert_lobs = auto_convert_lobs
+        if self.dbapi is not None:
+            self.ORACLE_BINARY_TYPES = [getattr(self.dbapi, k) for k in ["BFILE", "CLOB", "NCLOB", "BLOB", "LONG_BINARY", "LONG_STRING"] if hasattr(self.dbapi, k)]
+        else:
+            self.ORACLE_BINARY_TYPES = []
 
+    def dbapi(cls):
+        import cx_Oracle
+        return cx_Oracle
+    dbapi = classmethod(dbapi)
+    
     def create_connect_args(self, url):
         if url.database:
             # if we have a database, then we have a remote host
@@ -176,7 +226,7 @@ class OracleDialect(ansisql.ANSIDialect):
                 port = int(port)
             else:
                 port = 1521
-            dsn = self.module.makedsn(url.host,port,url.database)
+            dsn = self.dbapi.makedsn(url.host,port,url.database)
         else:
             # we have a local tnsname
             dsn = url.host
@@ -187,59 +237,123 @@ class OracleDialect(ansisql.ANSIDialect):
             threaded = self.threaded
             )
         opts.update(url.query)
+        util.coerce_kw_type(opts, 'use_ansi', bool)
         return ([], opts)
-        
+
     def type_descriptor(self, typeobj):
         return sqltypes.adapt_type(typeobj, colspecs)
 
-    def oid_column_name(self):
-        return "rowid"
+    def supports_unicode_statements(self):
+        """indicate whether the DBAPI can receive SQL statements as Python unicode strings"""
+        return False
 
-    def create_execution_context(self):
-        return OracleExecutionContext(self)
+    def max_identifier_length(self):
+        return 30
+        
+    def oid_column_name(self, column):
+        if not isinstance(column.table, sql.TableClause) and not isinstance(column.table, sql.Select):
+            return None
+        else:
+            return "rowid"
+
+    def create_execution_context(self, *args, **kwargs):
+        return OracleExecutionContext(self, *args, **kwargs)
 
     def compiler(self, statement, bindparams, **kwargs):
         return OracleCompiler(self, statement, bindparams, **kwargs)
+
     def schemagenerator(self, *args, **kwargs):
-        return OracleSchemaGenerator(*args, **kwargs)
+        return OracleSchemaGenerator(self, *args, **kwargs)
+
     def schemadropper(self, *args, **kwargs):
-        return OracleSchemaDropper(*args, **kwargs)
-    def defaultrunner(self, engine, proxy):
-        return OracleDefaultRunner(engine, proxy)
+        return OracleSchemaDropper(self, *args, **kwargs)
 
+    def defaultrunner(self, connection, **kwargs):
+        return OracleDefaultRunner(connection, **kwargs)
 
-    def has_table(self, connection, table_name):
+    def has_table(self, connection, table_name, schema=None):
         cursor = connection.execute("""select table_name from all_tables where table_name=:name""", {'name':table_name.upper()})
         return bool( cursor.fetchone() is not None )
 
     def has_sequence(self, connection, sequence_name):
         cursor = connection.execute("""select sequence_name from all_sequences where sequence_name=:name""", {'name':sequence_name.upper()})
         return bool( cursor.fetchone() is not None )
-        
+
+    def _locate_owner_row(self, owner, name, rows, raiseerr=False):
+        """return the row in the given list of rows which references the given table name and owner name."""
+        if not rows:
+            if raiseerr:
+                raise exceptions.NoSuchTableError(name)
+            else:
+                return None
+        else:
+            if owner is not None:
+                for row in rows:
+                    if owner.upper() in row[0]:
+                        return row
+                else:
+                    if raiseerr:
+                        raise exceptions.AssertionError("Specified owner %s does not own table %s" % (owner, name))
+                    else:
+                        return None
+            else:
+                if len(rows)==1:
+                    return rows[0]
+                else:
+                    if raiseerr:
+                        raise exceptions.AssertionError("There are multiple tables with name '%s' visible to the schema, you must specifiy owner" % name)
+                    else:
+                        return None
+
+    def _resolve_table_owner(self, connection, name, table, dblink=''):
+        """Locate the given table in the ``ALL_TAB_COLUMNS`` view,
+        including searching for equivalent synonyms and dblinks.
+        """
+
+        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS%(dblink)s where TABLE_NAME = :table_name" % {'dblink':dblink}, {'table_name':name})
+        rows = c.fetchall()
+        try:
+            row = self._locate_owner_row(table.owner, name, rows, raiseerr=True)
+            return name, row['OWNER'], ''
+        except exceptions.SQLAlchemyError:
+            # locate synonyms
+            c = connection.execute ("""select OWNER, TABLE_OWNER, TABLE_NAME, DB_LINK
+                                       from   ALL_SYNONYMS%(dblink)s
+                                       where  SYNONYM_NAME = :synonym_name
+                                       and (DB_LINK IS NOT NULL
+                                               or ((TABLE_NAME, TABLE_OWNER) in
+                                                    (select TABLE_NAME, OWNER from ALL_TAB_COLUMNS%(dblink)s)))""" % {'dblink':dblink},
+                                    {'synonym_name':name})
+            rows = c.fetchall()
+            row = self._locate_owner_row(table.owner, name, rows)
+            if row is None:
+                row = self._locate_owner_row("PUBLIC", name, rows)
+
+            if row is not None:
+                owner, name, dblink = row['TABLE_OWNER'], row['TABLE_NAME'], row['DB_LINK']
+                if dblink:
+                    dblink = '@' + dblink
+                    if not owner:
+                        # re-resolve table owner using new dblink variable
+                        t1, owner, t2 = self._resolve_table_owner(connection, name, table, dblink=dblink)
+                else:
+                    dblink = ''
+                return name, owner, dblink
+            raise
+
     def reflecttable(self, connection, table):
         preparer = self.identifier_preparer
         if not preparer.should_quote(table):
             name = table.name.upper()
         else:
             name = table.name
-        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS where TABLE_NAME = :table_name", {'table_name':name})
-        rows = c.fetchall()
-        if not rows :
-            raise exceptions.NoSuchTableError(table.name)
-        else:
-            if table.owner is not None:
-                if table.owner.upper() in [r[0] for r in rows]:
-                    owner = table.owner.upper()
-                else:
-                    raise exceptions.AssertionError("Specified owner %s does not own table %s"%(table.owner, table.name))
-            else:
-                if len(rows)==1:
-                    owner = rows[0][0]
-                else:
-                    raise exceptions.AssertionError("There are multiple tables with name %s in the schema, you must specifie owner"%table.name)
 
-        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS where TABLE_NAME = :table_name and OWNER = :owner", {'table_name':name, 'owner':owner})
-        
+        # search for table, including across synonyms and dblinks.
+        # locate the actual name of the table, the real owner, and any dblink clause needed.
+        actual_name, owner, dblink = self._resolve_table_owner(connection, name, table)
+
+        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS%(dblink)s where TABLE_NAME = :table_name and OWNER = :owner" % {'dblink':dblink}, {'table_name':actual_name, 'owner':owner})
+
         while True:
             row = c.fetchone()
             if row is None:
@@ -247,7 +361,7 @@ class OracleDialect(ansisql.ANSIDialect):
             found_table = True
 
             #print "ROW:" , row
-            (name, coltype, length, precision, scale, nullable, default) = (row[0], row[1], row[2], row[3], row[4], row[5]=='Y', row[6])
+            (colname, coltype, length, precision, scale, nullable, default) = (row[0], row[1], row[2], row[3], row[4], row[5]=='Y', row[6])
 
             # INTEGER if the scale is 0 and precision is null
             # NUMBER if the scale and precision are both null
@@ -268,26 +382,48 @@ class OracleDialect(ansisql.ANSIDialect):
                 try:
                     coltype = ischema_names[coltype]
                 except KeyError:
-                    raise exceptions.AssertionError("Cant get coltype for type '%s'" % coltype)
-               
+                    raise exceptions.AssertionError("Can't get coltype for type '%s' on colname '%s'" % (coltype, colname))
+
             colargs = []
             if default is not None:
                 colargs.append(schema.PassiveDefault(sql.text(default)))
-          
-            # if name comes back as all upper, assume its case folded 
-            if (name.upper() == name): 
-                name = name.lower()
-            
-            table.append_column(schema.Column(name, coltype, nullable=nullable, *colargs))
 
-       
-        c = connection.execute(constraintSQL, {'table_name' : table.name.upper(), 'owner' : owner})
+            # if name comes back as all upper, assume its case folded
+            if (colname.upper() == colname):
+                colname = colname.lower()
+
+            table.append_column(schema.Column(colname, coltype, nullable=nullable, *colargs))
+
+        if not len(table.columns):
+           raise exceptions.AssertionError("Couldn't find any column information for table %s" % actual_name)
+
+        c = connection.execute("""SELECT
+             ac.constraint_name,
+             ac.constraint_type,
+             LOWER(loc.column_name) AS local_column,
+             LOWER(rem.table_name) AS remote_table,
+             LOWER(rem.column_name) AS remote_column,
+             LOWER(rem.owner) AS remote_owner
+           FROM all_constraints%(dblink)s ac,
+             all_cons_columns%(dblink)s loc,
+             all_cons_columns%(dblink)s rem
+           WHERE ac.table_name = :table_name
+           AND ac.constraint_type IN ('R','P')
+           AND ac.owner = :owner
+           AND ac.owner = loc.owner
+           AND ac.constraint_name = loc.constraint_name
+           AND ac.r_owner = rem.owner(+)
+           AND ac.r_constraint_name = rem.constraint_name(+)
+           -- order multiple primary keys correctly
+           ORDER BY ac.constraint_name, loc.position, rem.position"""
+         % {'dblink':dblink}, {'table_name' : actual_name, 'owner' : owner})
+
         fks = {}
         while True:
             row = c.fetchone()
             if row is None:
                 break
-            #print "ROW:" , row                
+            #print "ROW:" , row
             (cons_name, cons_type, local_column, remote_table, remote_column, remote_owner) = row
             if cons_type == 'P':
                 table.primary_key.add(table.c[local_column])
@@ -297,6 +433,10 @@ class OracleDialect(ansisql.ANSIDialect):
                 except KeyError:
                    fk = ([], [])
                    fks[cons_name] = fk
+                if remote_table is None:
+                    # ticket 363
+                    warnings.warn("Got 'None' querying 'table_name' from all_cons_columns%(dblink)s - does the user have proper rights to the table?" % {'dblink':dblink})
+                    continue
                 refspec = ".".join([remote_table, remote_column])
                 schema.Table(remote_table, table.metadata, autoload=True, autoload_with=connection, owner=remote_owner)
                 if local_column not in fk[0]:
@@ -315,13 +455,26 @@ class OracleDialect(ansisql.ANSIDialect):
         if context is not None:
             context._rowcount = rowcount
 
+
+OracleDialect.logger = logging.class_logger(OracleDialect)
+
 class OracleCompiler(ansisql.ANSICompiler):
-    """oracle compiler modifies the lexical structure of Select statements to work under 
-    non-ANSI configured Oracle databases, if the use_ansi flag is False."""
-    
+    """Oracle compiler modifies the lexical structure of Select
+    statements to work under non-ANSI configured Oracle databases, if
+    the use_ansi flag is False.
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(OracleCompiler, self).__init__(*args, **kwargs)
+        # we have to modify SELECT objects a little bit, so store state here
+        self._select_state = {}
+        
     def default_from(self):
-        """called when a SELECT statement has no froms, and no FROM clause is to be appended.  
-        gives Oracle a chance to tack on a "FROM DUAL" to the string output. """
+        """Called when a ``SELECT`` statement has no froms, and no ``FROM`` clause is to be appended.
+
+        The Oracle compiler tacks a "FROM DUAL" to the statement.
+        """
+
         return " FROM DUAL"
 
     def apply_function_parens(self, func):
@@ -330,9 +483,14 @@ class OracleCompiler(ansisql.ANSICompiler):
     def visit_join(self, join):
         if self.dialect.use_ansi:
             return ansisql.ANSICompiler.visit_join(self, join)
-        
+
         self.froms[join] = self.get_from_text(join.left) + ", " + self.get_from_text(join.right)
-        self.wheres[join] = sql.and_(self.wheres.get(join.left, None), join.onclause)
+        where = self.wheres.get(join.left, None)
+        if where is not None:
+            self.wheres[join] = sql.and_(where, join.onclause)
+        else:
+            self.wheres[join] = join.onclause
+#        self.wheres[join] = sql.and_(self.wheres.get(join.left, None), join.onclause)
         self.strings[join] = self.froms[join]
 
         if join.isouter:
@@ -341,53 +499,66 @@ class OracleCompiler(ansisql.ANSICompiler):
 
             # now re-visit the onclause, which will be used as a where clause
             # (the first visit occured via the Join object itself right before it called visit_join())
-            join.onclause.accept_visitor(self)
+            self.traverse(join.onclause)
 
             self._outertable = None
 
-        self.visit_compound(self.wheres[join])
-       
+        self.wheres[join].accept_visitor(self)
+
+    def visit_insert_sequence(self, column, sequence, parameters):
+        """This is the `sequence` equivalent to ``ANSICompiler``'s
+        `visit_insert_column_default` which ensures that the column is
+        present in the generated column list.
+        """
+
+        parameters.setdefault(column.key, None)
+
     def visit_alias(self, alias):
-	"""oracle doesnt like 'FROM table AS alias'.  is the AS standard SQL??"""
+        """Oracle doesn't like ``FROM table AS alias``.  Is the AS standard SQL??"""
+
         self.froms[alias] = self.get_from_text(alias.original) + " " + alias.name
         self.strings[alias] = self.get_str(alias.original)
- 
+
     def visit_column(self, column):
         ansisql.ANSICompiler.visit_column(self, column)
         if not self.dialect.use_ansi and getattr(self, '_outertable', None) is not None and column.table is self._outertable:
             self.strings[column] = self.strings[column] + "(+)"
-       
+
     def visit_insert(self, insert):
-        """inserts are required to have the primary keys be explicitly present.
-         mapper will by default not put them in the insert statement to comply
-         with autoincrement fields that require they not be present.  so, 
-         put them all in for all primary key columns."""
+        """``INSERT`` s are required to have the primary keys be explicitly present.
+
+         Mapper will by default not put them in the insert statement
+         to comply with autoincrement fields that require they not be
+         present.  so, put them all in for all primary key columns.
+         """
+
         for c in insert.table.primary_key:
             if not self.parameters.has_key(c.key):
                 self.parameters[c.key] = None
         return ansisql.ANSICompiler.visit_insert(self, insert)
 
     def _TODO_visit_compound_select(self, select):
-        """need to determine how to get LIMIT/OFFSET into a UNION for oracle"""
+        """Need to determine how to get ``LIMIT``/``OFFSET`` into a ``UNION`` for Oracle."""
+
         if getattr(select, '_oracle_visit', False):
             # cancel out the compiled order_by on the select
             if hasattr(select, "order_by_clause"):
                 self.strings[select.order_by_clause] = ""
             ansisql.ANSICompiler.visit_compound_select(self, select)
             return
-            
+
         if select.limit is not None or select.offset is not None:
             select._oracle_visit = True
-            # to use ROW_NUMBER(), an ORDER BY is required. 
+            # to use ROW_NUMBER(), an ORDER BY is required.
             orderby = self.strings[select.order_by_clause]
             if not orderby:
                 orderby = select.oid_column
-                orderby.accept_visitor(self)
+                self.traverse(orderby)
                 orderby = self.strings[orderby]
-            class SelectVisitor(sql.ClauseVisitor):
+            class SelectVisitor(sql.NoColumnVisitor):
                 def visit_select(self, select):
-                    select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
-            select.accept_visitor(SelectVisitor())
+                    select.append_column(sql.literal_column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+            SelectVisitor().traverse(select)
             limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
             if select.offset is not None:
                 limitselect.append_whereclause("ora_rn>%d" % select.offset)
@@ -395,18 +566,20 @@ class OracleCompiler(ansisql.ANSICompiler):
                     limitselect.append_whereclause("ora_rn<=%d" % (select.limit + select.offset))
             else:
                 limitselect.append_whereclause("ora_rn<=%d" % select.limit)
-            limitselect.accept_visitor(self)
+            self.traverse(limitselect)
             self.strings[select] = self.strings[limitselect]
             self.froms[select] = self.froms[limitselect]
         else:
             ansisql.ANSICompiler.visit_compound_select(self, select)
-        
+
     def visit_select(self, select):
-        """looks for LIMIT and OFFSET in a select statement, and if so tries to wrap it in a 
-        subquery with row_number() criterion."""
+        """Look for ``LIMIT`` and OFFSET in a select statement, and if
+        so tries to wrap it in a subquery with ``row_number()`` criterion.
+        """
+
         # TODO: put a real copy-container on Select and copy, or somehow make this
         # not modify the Select statement
-        if getattr(select, '_oracle_visit', False):
+        if self._select_state.get((select, 'visit'), False):
             # cancel out the compiled order_by on the select
             if hasattr(select, "order_by_clause"):
                 self.strings[select.order_by_clause] = ""
@@ -414,14 +587,16 @@ class OracleCompiler(ansisql.ANSICompiler):
             return
 
         if select.limit is not None or select.offset is not None:
-            select._oracle_visit = True
-            # to use ROW_NUMBER(), an ORDER BY is required. 
+            self._select_state[(select, 'visit')] = True
+            # to use ROW_NUMBER(), an ORDER BY is required.
             orderby = self.strings[select.order_by_clause]
             if not orderby:
                 orderby = select.oid_column
-                orderby.accept_visitor(self)
+                self.traverse(orderby)
                 orderby = self.strings[orderby]
-            select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+            if not hasattr(select, '_oracle_visit'):
+                select.append_column(sql.literal_column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+                select._oracle_visit = True
             limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
             if select.offset is not None:
                 limitselect.append_whereclause("ora_rn>%d" % select.offset)
@@ -429,12 +604,12 @@ class OracleCompiler(ansisql.ANSICompiler):
                     limitselect.append_whereclause("ora_rn<=%d" % (select.limit + select.offset))
             else:
                 limitselect.append_whereclause("ora_rn<=%d" % select.limit)
-            limitselect.accept_visitor(self)
+            self.traverse(limitselect)
             self.strings[select] = self.strings[limitselect]
             self.froms[select] = self.froms[limitselect]
         else:
             ansisql.ANSICompiler.visit_select(self, select)
-            
+
     def limit_clause(self, select):
         return ""
 
@@ -444,10 +619,17 @@ class OracleCompiler(ansisql.ANSICompiler):
         else:
             return super(OracleCompiler, self).for_update_clause(select)
 
+    def visit_binary(self, binary):
+        if binary.operator == '%': 
+            self.strings[binary] = ("MOD(%s,%s)"%(self.get_str(binary.left), self.get_str(binary.right)))
+        else:
+            return ansisql.ANSICompiler.visit_binary(self, binary)
+        
+
 class OracleSchemaGenerator(ansisql.ANSISchemaGenerator):
     def get_column_specification(self, column, **kwargs):
         colspec = self.preparer.format_column(column)
-        colspec += " " + column.type.engine_impl(self.engine).get_col_spec()
+        colspec += " " + column.type.dialect_impl(self.dialect).get_col_spec()
         default = self.get_column_default_string(column)
         if default is not None:
             colspec += " DEFAULT " + default
@@ -457,22 +639,22 @@ class OracleSchemaGenerator(ansisql.ANSISchemaGenerator):
         return colspec
 
     def visit_sequence(self, sequence):
-        if not self.engine.dialect.has_sequence(self.connection, sequence.name):
+        if not self.dialect.has_sequence(self.connection, sequence.name):
             self.append("CREATE SEQUENCE %s" % self.preparer.format_sequence(sequence))
             self.execute()
 
 class OracleSchemaDropper(ansisql.ANSISchemaDropper):
     def visit_sequence(self, sequence):
-        if self.engine.dialect.has_sequence(self.connection, sequence.name):
+        if self.dialect.has_sequence(self.connection, sequence.name):
             self.append("DROP SEQUENCE %s" % sequence.name)
             self.execute()
 
 class OracleDefaultRunner(ansisql.ANSIDefaultRunner):
     def exec_default_sql(self, default):
-        c = sql.select([default.arg], from_obj=["DUAL"], engine=self.engine).compile()
-        return self.proxy(str(c), c.get_params()).fetchone()[0]
-    
+        c = sql.select([default.arg], from_obj=["DUAL"]).compile(engine=self.connection)
+        return self.connection.execute_compiled(c).scalar()
+
     def visit_sequence(self, seq):
-        return self.proxy("SELECT " + seq.name + ".nextval FROM DUAL").fetchone()[0]
+        return self.connection.execute_text("SELECT " + seq.name + ".nextval FROM DUAL").scalar()
 
 dialect = OracleDialect
