@@ -1,6 +1,6 @@
 from testbase import PersistTest
 import testbase
-
+import pickle
 import sqlalchemy.ansisql as ansisql
 
 from sqlalchemy import *
@@ -11,8 +11,6 @@ import unittest, re, StringIO
 
 class ReflectionTest(PersistTest):
     def testbasic(self):
-        # really trip it up with a circular reference
-        
         use_function_defaults = testbase.db.engine.name == 'postgres' or testbase.db.engine.name == 'oracle'
         
         use_string_defaults = use_function_defaults or testbase.db.engine.__module__.endswith('sqlite')
@@ -27,11 +25,16 @@ class ReflectionTest(PersistTest):
         if use_string_defaults:
             deftype2 = String
             defval2 = "im a default"
+            #deftype3 = DateTime
+            # the colon thing isnt working out for PG reflection just yet
+            #defval3 = '1999-09-09 00:00:00'
+            deftype3 = Date
+            defval3 = '1999-09-09'
         else:
-            deftype2 = Integer
-            defval2 = "15"
+            deftype2, deftype3 = Integer, Integer
+            defval2, defval3 = "15", "16"
         
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         
         users = Table('engine_users', meta,
             Column('user_id', INT, primary_key = True),
@@ -48,6 +51,7 @@ class ReflectionTest(PersistTest):
             Column('test_passivedefault', deftype, PassiveDefault(defval)),
             Column('test_passivedefault2', Integer, PassiveDefault("5")),
             Column('test_passivedefault3', deftype2, PassiveDefault(defval2)),
+            Column('test_passivedefault4', deftype3, PassiveDefault(defval3)),
             Column('test9', Binary(100)),
             Column('test_numeric', Numeric(None, None)),
             mysql_engine='InnoDB'
@@ -60,7 +64,7 @@ class ReflectionTest(PersistTest):
             mysql_engine='InnoDB'
         )
         meta.drop_all()
-        
+
         users.create()
         addresses.create()
 
@@ -72,6 +76,8 @@ class ReflectionTest(PersistTest):
             # reference the addresses foreign key col, which will require users to be 
             # reflected at some point
             users = Table('engine_users', meta, autoload = True)
+            assert users.c.user_id in users.primary_key
+            assert len(users.primary_key) == 1
         finally:
             addresses.drop()
             users.drop()
@@ -85,7 +91,7 @@ class ReflectionTest(PersistTest):
         users.create()
         addresses.create()
         try:
-            # create a join from the two tables, this insures that
+            # create a join from the two tables, this ensures that
             # theres a foreign key set up
             # previously, we couldnt get foreign keys out of mysql.  seems like
             # we can now as long as we use InnoDB
@@ -99,27 +105,10 @@ class ReflectionTest(PersistTest):
         finally:
             addresses.drop()
             users.drop()
-            
-    @testbase.supported('postgres')
-    def testpgdates(self):
-        m1 = BoundMetaData(testbase.db)
-        t1 = Table('pgdate', m1, 
-            Column('date1', DateTime(timezone=True)),
-            Column('date2', DateTime(timezone=False))
-            )
-        m1.create_all()
-        try:
-            m2 = BoundMetaData(testbase.db)
-            t2 = Table('pgdate', m2, autoload=True)
-            assert t2.c.date1.type.timezone is True
-            assert t2.c.date2.type.timezone is False
-        finally:
-            m1.drop_all()
-            
-            
+    
     def testoverridecolumns(self):
         """test that you can override columns which contain foreign keys to other reflected tables"""
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         users = Table('users', meta, 
             Column('id', Integer, primary_key=True),
             Column('name', String(30)))
@@ -130,7 +119,7 @@ class ReflectionTest(PersistTest):
             
         meta.create_all()            
         try:
-            meta2 = BoundMetaData(testbase.db)
+            meta2 = MetaData(testbase.db)
             a2 = Table('addresses', meta2, 
                 Column('user_id', Integer, ForeignKey('users.id')),
                 autoload=True)
@@ -140,21 +129,38 @@ class ReflectionTest(PersistTest):
             assert list(a2.c.user_id.foreign_keys)[0].parent is a2.c.user_id
             assert u2.join(a2).onclause == u2.c.id==a2.c.user_id
 
-            meta3 = BoundMetaData(testbase.db)
+            meta3 = MetaData(testbase.db)
             u3 = Table('users', meta3, autoload=True)
             a3 = Table('addresses', meta3, 
                 Column('user_id', Integer, ForeignKey('users.id')),
                 autoload=True)
             
             assert u3.join(a3).onclause == u3.c.id==a3.c.user_id
-            
+
+            meta4 = MetaData(testbase.db)
+            u4 = Table('users', meta4,
+                       Column('id', Integer, key='u_id', primary_key=True),
+                       autoload=True)
+            a4 = Table('addresses', meta4,
+                       Column('id', Integer, key='street', primary_key=True),
+                       Column('street', String(30), key='user_id'),
+                       Column('user_id', Integer, ForeignKey('users.u_id'),
+                              key='id'),
+                       autoload=True)
+
+            assert u4.join(a4).onclause.compare(u4.c.u_id==a4.c.id)
+            assert list(u4.primary_key) == [u4.c.u_id]
+            assert len(u4.columns) == 2
+            assert len(u4.constraints) == 1
+            assert len(a4.columns) == 3
+            assert len(a4.constraints) == 2
         finally:
             meta.drop_all()
 
     def testoverridecolumns2(self):
         """test that you can override columns which contain foreign keys to other reflected tables,
         where the foreign key column is also a primary key column"""
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         users = Table('users', meta, 
             Column('id', Integer, primary_key=True),
             Column('name', String(30)))
@@ -165,7 +171,7 @@ class ReflectionTest(PersistTest):
 
         meta.create_all()            
         try:
-            meta2 = BoundMetaData(testbase.db)
+            meta2 = MetaData(testbase.db)
             a2 = Table('addresses', meta2, 
                 Column('id', Integer, ForeignKey('users.id'), primary_key=True, ),
                 autoload=True)
@@ -184,7 +190,7 @@ class ReflectionTest(PersistTest):
             #sess.save(add1)
             #sess.flush()
             
-            meta3 = BoundMetaData(testbase.db)
+            meta3 = MetaData(testbase.db)
             u3 = Table('users', meta3, autoload=True)
             a3 = Table('addresses', meta3, 
                 Column('id', Integer, ForeignKey('users.id'), primary_key=True),
@@ -199,22 +205,23 @@ class ReflectionTest(PersistTest):
             
     @testbase.supported('mysql')
     def testmysqltypes(self):
-        meta1 = BoundMetaData(testbase.db)
+        meta1 = MetaData(testbase.db)
         table = Table(
             'mysql_types', meta1,
             Column('id', Integer, primary_key=True),
             Column('num1', mysql.MSInteger(unsigned=True)),
             Column('text1', mysql.MSLongText),
             Column('text2', mysql.MSLongText()),
-             Column('num2', mysql.MSBigInteger),
-             Column('num3', mysql.MSBigInteger()),
-             Column('num4', mysql.MSDouble),
-             Column('num5', mysql.MSDouble()),
-             Column('enum1', mysql.MSEnum('"black"', '"white"')),
+            Column('num2', mysql.MSBigInteger),
+            Column('num3', mysql.MSBigInteger()),
+            Column('num4', mysql.MSDouble),
+            Column('num5', mysql.MSDouble()),
+            Column('enum1', mysql.MSEnum('"black"', '"white"')),
             )
         try:
-            table.create(checkfirst=True)
-            meta2 = BoundMetaData(testbase.db)
+            table.drop(checkfirst=True)
+            table.create()
+            meta2 = MetaData(testbase.db)
             t2 = Table('mysql_types', meta2, autoload=True)
             assert isinstance(t2.c.num1.type, mysql.MSInteger)
             assert t2.c.num1.type.unsigned
@@ -230,19 +237,52 @@ class ReflectionTest(PersistTest):
         finally:
             table.drop(checkfirst=True)
             
-    @testbase.supported('postgres')
-    def testredundantsequence(self):
-        meta1 = BoundMetaData(testbase.db)
-        t = Table('mytable', meta1, 
-            Column('col1', Integer, Sequence('fooseq')))
+    
+    def test_pks_not_uniques(self):
+        """test that primary key reflection not tripped up by unique indexes"""
+        testbase.db.execute("""
+        CREATE TABLE book (
+            id INTEGER NOT NULL,
+            title VARCHAR(100) NOT NULL,
+            series INTEGER NULL,
+            series_id INTEGER NULL,
+            UNIQUE(series, series_id),
+            PRIMARY KEY(id)
+        )""")
         try:
-            testbase.db.execute("CREATE SEQUENCE fooseq")
-            t.create()
+            metadata = MetaData(engine=testbase.db)
+            book = Table('book', metadata, autoload=True)
+            assert book.c.id  in book.primary_key
+            assert book.c.series not in book.primary_key
+            assert len(book.primary_key) == 1
         finally:
-            t.drop()
+            testbase.db.execute("drop table book")
+
+    def test_composite_pks(self):
+        """test reflection of a composite primary key"""
+        testbase.db.execute("""
+        CREATE TABLE book (
+            id INTEGER NOT NULL,
+            isbn VARCHAR(50) NOT NULL,
+            title VARCHAR(100) NOT NULL,
+            series INTEGER NULL,
+            series_id INTEGER NULL,
+            UNIQUE(series, series_id),
+            PRIMARY KEY(id, isbn)
+        )""")
+        try:
+            metadata = MetaData(engine=testbase.db)
+            book = Table('book', metadata, autoload=True)
+            assert book.c.id  in book.primary_key
+            assert book.c.isbn  in book.primary_key
+            assert book.c.series not in book.primary_key
+            assert len(book.primary_key) == 2
+        finally:
+            testbase.db.execute("drop table book")
             
     @testbase.supported('sqlite')
     def test_goofy_sqlite(self):
+        """test autoload of table where quotes were used with all the colnames.  quirky in sqlite."""
         testbase.db.execute("""CREATE TABLE "django_content_type" (
             "id" integer NOT NULL PRIMARY KEY,
             "django_stuff" text NULL
@@ -258,7 +298,7 @@ class ReflectionTest(PersistTest):
         )
         """)
         try:
-            meta = BoundMetaData(testbase.db)
+            meta = MetaData(testbase.db)
             table1 = Table("django_admin_log", meta, autoload=True)
             table2 = Table("django_content_type", meta, autoload=True)
             j = table1.join(table2)
@@ -267,40 +307,9 @@ class ReflectionTest(PersistTest):
             testbase.db.execute("drop table django_admin_log")
             testbase.db.execute("drop table django_content_type")
 
-    def testmultipk(self):
-        """test that creating a table checks for a sequence before creating it"""
-        meta = BoundMetaData(testbase.db)
-        table = Table(
-            'engine_multi', meta, 
-            Column('multi_id', Integer, Sequence('multi_id_seq'), primary_key=True),
-            Column('multi_rev', Integer, Sequence('multi_rev_seq'), primary_key=True),
-            Column('name', String(50), nullable=False),
-            Column('val', String(100))
-        )
-        table.create()
-
-        meta2 = BoundMetaData(testbase.db)
-        try:
-            table = Table('engine_multi', meta2, autoload=True)
-        finally:
-            table.drop()
-        
-        print repr(
-            [table.c['multi_id'].primary_key,
-            table.c['multi_rev'].primary_key
-            ]
-        )
-
-
-        table.create()
-        table.insert().execute({'multi_id':1,'multi_rev':1,'name':'row1', 'val':'value1'})
-        table.insert().execute({'multi_id':2,'multi_rev':18,'name':'row2', 'val':'value2'})
-        table.insert().execute({'multi_id':3,'multi_rev':3,'name':'row3', 'val':'value3'})
-        table.select().execute().fetchall()
-        table.drop()
-
-    def testcompositefk(self):
-        meta = BoundMetaData(testbase.db)
+    def test_composite_fk(self):
+        """test reflection of composite foreign keys"""
+        meta = MetaData(testbase.db)
         table = Table(
             'multi', meta, 
             Column('multi_id', Integer, primary_key=True),
@@ -336,53 +345,79 @@ class ReflectionTest(PersistTest):
         finally:
             meta.drop_all()
 
-    def testcheckfirst(self):
-        meta = BoundMetaData(testbase.db)
-        
-        table = Table('checkfirst', meta, 
-            Column('col1', Integer, primary_key=True),
-            Column('col2', String(40)))
-        try:
-            assert not table.exists()
-            table.create()
-            assert table.exists()
-            table.create(checkfirst=True)
-            table.drop()
-            table.drop(checkfirst=True)
-            assert not table.exists()
-            table.create(checkfirst=True)
-            table.drop()
-        finally:
-            meta.drop_all()
             
-    def testtoengine(self):
-        meta = MetaData('md1')
-        meta2 = MetaData('md2')
+    def test_to_metadata(self):
+        meta = MetaData()
         
         table = Table('mytable', meta,
             Column('myid', Integer, primary_key=True),
-            Column('name', String, nullable=False),
-            Column('description', String(30)),
+            Column('name', String(40), nullable=False),
+            Column('description', String(30), CheckConstraint("description='hi'")),
+            UniqueConstraint('name'),
+            mysql_engine='InnoDB'
         )
         
         table2 = Table('othertable', meta,
             Column('id', Integer, primary_key=True),
-            Column('myid', Integer, ForeignKey('mytable.myid'))
+            Column('myid', Integer, ForeignKey('mytable.myid')),
+            mysql_engine='InnoDB'
             )
+        
+        def test_to_metadata():
+            meta2 = MetaData()
+            table_c = table.tometadata(meta2)
+            table2_c = table2.tometadata(meta2)
+            return (table_c, table2_c)
             
-        
-        table_c = table.tometadata(meta2)
-        table2_c = table2.tometadata(meta2)
+        def test_pickle():
+            meta.connect(testbase.db)
+            meta2 = pickle.loads(pickle.dumps(meta))
+            assert meta2.engine is None
+            return (meta2.tables['mytable'], meta2.tables['othertable'])
 
-        assert table is not table_c
-        assert table_c.c.myid.primary_key
-        assert not table_c.c.name.nullable 
-        assert table_c.c.description.nullable 
-        assert table.primary_key is not table_c.primary_key
-        assert [x.name for x in table.primary_key] == [x.name for x in table_c.primary_key]
-        assert list(table2_c.c.myid.foreign_keys)[0].column is table_c.c.myid
-        assert list(table2_c.c.myid.foreign_keys)[0].column is not table.c.myid
+        def test_pickle_via_reflect():
+            # this is the most common use case, pickling the results of a
+            # database reflection
+            meta2 = MetaData(engine=testbase.db)
+            t1 = Table('mytable', meta2, autoload=True)
+            t2 = Table('othertable', meta2, autoload=True)
+            meta3 = pickle.loads(pickle.dumps(meta2))
+            assert meta3.engine is None
+            assert meta3.tables['mytable'] is not t1
+            return (meta3.tables['mytable'], meta3.tables['othertable'])
+            
+        meta.create_all(testbase.db)    
+        try:
+            for test, has_constraints in ((test_to_metadata, True), (test_pickle, True), (test_pickle_via_reflect, False)):
+                table_c, table2_c = test()
+                assert table is not table_c
+                assert table_c.c.myid.primary_key
+                assert not table_c.c.name.nullable 
+                assert table_c.c.description.nullable 
+                assert table.primary_key is not table_c.primary_key
+                assert [x.name for x in table.primary_key] == [x.name for x in table_c.primary_key]
+                assert list(table2_c.c.myid.foreign_keys)[0].column is table_c.c.myid
+                assert list(table2_c.c.myid.foreign_keys)[0].column is not table.c.myid
+                
+                # constraints dont get reflected for any dialect right now
+                if has_constraints:
+                    for c in table_c.c.description.constraints:
+                        if isinstance(c, CheckConstraint):
+                            break
+                    else:
+                        assert False
+                    assert c.sqltext=="description='hi'"
         
+                    for c in table_c.constraints:
+                        if isinstance(c, UniqueConstraint):
+                            break
+                    else:
+                        assert False
+                    assert c.columns.contains_column(table_c.c.name)
+                    assert not c.columns.contains_column(table.c.name)
+        finally:
+            meta.drop_all(testbase.db)
+            
     # mysql throws its own exception for no such table, resulting in 
     # a sqlalchemy.SQLError instead of sqlalchemy.NoSuchTableError.
     # this could probably be fixed at some point.
@@ -393,7 +428,7 @@ class ReflectionTest(PersistTest):
                           testbase.db, autoload=True)
         
     def testoverride(self):
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         table = Table(
             'override_test', meta, 
             Column('col1', Integer, primary_key=True),
@@ -403,7 +438,7 @@ class ReflectionTest(PersistTest):
         table.create()
         # clear out table registry
 
-        meta2 = BoundMetaData(testbase.db)
+        meta2 = MetaData(testbase.db)
         try:
             table = Table(
                 'override_test', meta2,
@@ -417,9 +452,76 @@ class ReflectionTest(PersistTest):
         finally:
             table.drop()
 
+    @testbase.supported('mssql')
+    def testidentity(self):
+        meta = MetaData(testbase.db)
+        table = Table(
+            'identity_test', meta, 
+            Column('col1', Integer, Sequence('fred', 2, 3), primary_key=True)
+        )
+        table.create()
+        
+        meta2 = MetaData(testbase.db)
+        try:
+            table2 = Table('identity_test', meta2, autoload=True)
+            assert table2.c['col1'].sequence.start == 2
+            assert table2.c['col1'].sequence.increment == 3
+        finally:
+            table.drop()
+
+    def testreserved(self):
+        # check a table that uses an SQL reserved name doesn't cause an error
+        meta = MetaData(testbase.db)
+        table_a = Table('select', meta, 
+                       Column('not', Integer, primary_key=True),
+                       Column('from', String(12), nullable=False),
+                       UniqueConstraint('from', name='when'))
+        Index('where', table_a.c['from'])
+
+        quoter = meta.bind.dialect.identifier_preparer.quote_identifier
+
+        table_b = Table('false', meta,
+                        Column('create', Integer, primary_key=True),
+                        Column('true', Integer, ForeignKey('select.not')),
+                        CheckConstraint('%s <> 1' % quoter('true'), name='limit'))
+
+        table_c = Table('is', meta,
+                        Column('or', Integer, nullable=False, primary_key=True),
+                        Column('join', Integer, nullable=False, primary_key=True),
+                        PrimaryKeyConstraint('or', 'join', name='to'))
+
+        index_c = Index('else', table_c.c.join)
+
+        #meta.bind.echo = True
+        meta.create_all()
+
+        index_c.drop()
+        
+        meta2 = MetaData(testbase.db)
+        try:
+            table_a2 = Table('select', meta2, autoload=True)
+            table_b2 = Table('false', meta2, autoload=True)
+            table_c2 = Table('is', meta2, autoload=True)
+        finally:
+            meta.drop_all()
+
+
+        meta = MetaData(testbase.db)
+        table = Table(
+            'select', meta, 
+            Column('col1', Integer, primary_key=True)
+        )
+        table.create()
+        
+        meta2 = MetaData(testbase.db)
+        try:
+            table2 = Table('select', meta2, autoload=True)
+        finally:
+            table.drop()
+
 class CreateDropTest(PersistTest):
     def setUpAll(self):
-        global metadata
+        global metadata, users
         metadata = MetaData()
         users = Table('users', metadata,
                       Column('user_id', Integer, Sequence('user_id_seq', optional=True), primary_key = True),
@@ -453,6 +555,19 @@ class CreateDropTest(PersistTest):
         table_names = [t.name for t in tables]
         self.assert_( table_names == ['users', 'orders', 'items', 'email_addresses'] or table_names ==  ['users', 'email_addresses', 'orders', 'items'])
 
+    def testcheckfirst(self):
+        try:
+            assert not users.exists(testbase.db)
+            users.create(connectable=testbase.db)
+            assert users.exists(testbase.db)
+            users.create(connectable=testbase.db, checkfirst=True)
+            users.drop(connectable=testbase.db)
+            users.drop(connectable=testbase.db, checkfirst=True)
+            assert not users.exists(connectable=testbase.db)
+            users.create(connectable=testbase.db, checkfirst=True)
+            users.drop(connectable=testbase.db)
+        finally:
+            metadata.drop_all(connectable=testbase.db)
 
     def test_createdrop(self):
         metadata.create_all(connectable=testbase.db)
@@ -479,21 +594,44 @@ class SchemaTest(PersistTest):
             Column('col1', Integer, primary_key=True),
             Column('col2', Integer, ForeignKey('someschema.table1.col1')),
             schema='someschema')
-        # insure this doesnt crash
+        # ensure this doesnt crash
         print [t for t in metadata.table_iterator()]
         buf = StringIO.StringIO()
-        def foo(s, p):
+        def foo(s, p=None):
             buf.write(s)
-        gen = testbase.db.dialect.schemagenerator(testbase.db.engine, foo, None)
-        table1.accept_schema_visitor(gen)
-        table2.accept_schema_visitor(gen)
+        gen = create_engine(testbase.db.name + "://", strategy="mock", executor=foo)
+        gen = gen.dialect.schemagenerator(gen)
+        gen.traverse(table1)
+        gen.traverse(table2)
         buf = buf.getvalue()
         print buf
         assert buf.index("CREATE TABLE someschema.table1") > -1
         assert buf.index("CREATE TABLE someschema.table2") > -1
-         
-        
-        
+
+    @testbase.unsupported('sqlite', 'postgres')
+    def test_create_with_defaultschema(self):
+        engine = testbase.db
+        schema = engine.dialect.get_default_schema_name(engine)
+
+        # test reflection of tables with an explcit schemaname
+        # matching the default
+        metadata = MetaData(testbase.db)
+        table1 = Table('table1', metadata, 
+            Column('col1', Integer, primary_key=True),
+            schema=schema)
+        table2 = Table('table2', metadata, 
+            Column('col1', Integer, primary_key=True),
+            Column('col2', Integer, ForeignKey('%s.table1.col1' % schema)),
+            schema=schema)
+        metadata.create_all()
+        metadata.create_all(checkfirst=True)
+        metadata.clear()
+        table1 = Table('table1', metadata, autoload=True, schema=schema)
+        table2 = Table('table2', metadata, autoload=True, schema=schema)
+        assert table1.schema == table2.schema == schema
+        assert len(metadata.tables) == 2
+        metadata.drop_all()
+    
         
 if __name__ == "__main__":
     testbase.main()        
