@@ -2,7 +2,11 @@ import testbase
 import unittest, doctest
 
 def suite():
-    unittest_modules = ['ext.activemapper', 'ext.selectresults']
+    unittest_modules = ['ext.activemapper',
+                        'ext.selectresults',
+                        'ext.assignmapper',
+                        'ext.orderinglist',
+                        'ext.associationproxy']
     doctest_modules = ['sqlalchemy.ext.sqlsoup']
 
     alltests = unittest.TestSuite()
@@ -17,4 +21,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.runTests(suite())
+    testbase.main(suite())
