@@ -5,6 +5,10 @@ def suite():
     modules_to_test = (
 	'orm.attributes',
         'orm.mapper',
+        'orm.query',
+        'orm.lazy_relations',
+        'orm.eager_relations',
+        'orm.generative',
         'orm.lazytest1',
         'orm.eagertest1',
         'orm.eagertest2',
@@ -17,6 +21,7 @@ def suite():
         'orm.relationships',
         'orm.association',
         'orm.merge',
+        'orm.memusage',
         
         'orm.cycles',
         'orm.poly_linked_list',
@@ -30,6 +35,7 @@ def suite():
         'orm.inheritance3',
         'orm.inheritance4',
         'orm.inheritance5',
+        'orm.abc_inheritance',
         'orm.single',
         'orm.polymorph'        
         )
@@ -43,4 +49,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.runTests(suite())
+    testbase.main(suite())
