@@ -1,4 +1,4 @@
-from testbase import PersistTest, AssertMixin
+from testbase import PersistTest, AssertMixin, ORMTest
 import unittest, sys, os
 from sqlalchemy import *
 import StringIO
@@ -8,7 +8,12 @@ from tables import *
 import tables
 
 """test cyclical mapper relationships.  Many of the assertions are provided
-via running with postgres, which is strict about foreign keys."""
+via running with postgres, which is strict about foreign keys.
+
+we might want to try an automated generate of much of this, all combos of T1<->T2, with 
+o2m or m2o between them, and a third T3 with o2m/m2o to one/both T1/T2.
+"""
+
 
 class Tester(object):
     def __init__(self, data=None):
@@ -21,7 +26,7 @@ class SelfReferentialTest(AssertMixin):
     """tests a self-referential mapper, with an additional list of child objects."""
     def setUpAll(self):
         global t1, t2, metadata
-        metadata = BoundMetaData(testbase.db)
+        metadata = MetaData(testbase.db)
         t1 = Table('t1', metadata, 
             Column('c1', Integer, Sequence('t1c1_id_seq', optional=True), primary_key=True),
             Column('parent_c1', Integer, ForeignKey('t1.c1')),
@@ -43,7 +48,7 @@ class SelfReferentialTest(AssertMixin):
             pass
         m1 = mapper(C1, t1, properties = {
             'c1s':relation(C1, cascade="all"),
-            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, foreignkey=t1.c.c1, lazy=True, uselist=False)
+            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, remote_side=t1.c.c1, lazy=True, uselist=False)
         })
         a = C1('head c1')
         a.c1s.append(C1('another c1'))
@@ -64,7 +69,7 @@ class SelfReferentialTest(AssertMixin):
         class C1(Tester):
             pass
         mapper(C1, t1, properties={
-            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, foreignkey=t1.c.c1)
+            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, remote_side=t1.c.c1)
         })
         sess = create_session()
         c1 = C1()
@@ -118,11 +123,57 @@ class SelfReferentialTest(AssertMixin):
             assert False
         except exceptions.ArgumentError:
             assert True
-            
+
+class SelfReferentialNoPKTest(AssertMixin):
+    """test self-referential relationship that joins on a column other than the primary key column"""
+    def setUpAll(self):
+        global table, meta
+        meta = MetaData(testbase.db)
+        table = Table('item', meta,
+           Column('id', Integer, primary_key=True),
+           Column('uuid', String(32), unique=True, nullable=False),
+           Column('parent_uuid', String(32), ForeignKey('item.uuid'), nullable=True),
+        )
+        meta.create_all()
+    def tearDown(self):
+        table.delete().execute()
+    def tearDownAll(self):
+        meta.drop_all()
+    def testbasic(self):
+        class TT(object):
+            def __init__(self):
+                self.uuid = hex(id(self))
+        mapper(TT, table, properties={'children':relation(TT, remote_side=[table.c.parent_uuid], backref=backref('parent', remote_side=[table.c.uuid]))})
+        s = create_session()
+        t1 = TT()
+        t1.children.append(TT())
+        t1.children.append(TT())
+        s.save(t1)
+        s.flush()
+        s.clear()
+        t = s.query(TT).get_by(id=t1.id)
+        assert t.children[0].parent_uuid == t1.uuid
+    def testlazyclause(self):
+        class TT(object):
+            def __init__(self):
+                self.uuid = hex(id(self))
+        mapper(TT, table, properties={'children':relation(TT, remote_side=[table.c.parent_uuid], backref=backref('parent', remote_side=[table.c.uuid]))})
+        s = create_session()
+        t1 = TT()
+        t2 = TT()
+        t1.children.append(t2)
+        s.save(t1)
+        s.flush()
+        s.clear()
+
+        t = s.query(TT).get_by(id=t2.id)
+        assert t.uuid == t2.uuid
+        assert t.parent.uuid == t1.uuid
+        
 class InheritTestOne(AssertMixin):
     def setUpAll(self):
         global parent, child1, child2, meta
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         parent = Table("parent", meta,
             Column("id", Integer, primary_key=True),
             Column("parent_data", String(50)),
@@ -179,12 +230,152 @@ class InheritTestOne(AssertMixin):
         # the flush will fail if the UOW does not set up a many-to-one DP 
         # attached to a task corresponding to c1, since "child1_id" is not nullable
         session.flush()
+
+class InheritTestTwo(ORMTest):
+    """the fix in BiDirectionalManyToOneTest raised this issue, regarding
+    the 'circular sort' containing UOWTasks that were still polymorphic, which could
+    create duplicate entries in the final sort"""
+    def define_tables(self, metadata):
+        global a, b, c
+        a = Table('a', metadata, 
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            Column('cid', Integer, ForeignKey('c.id')),
+            )
+
+        b = Table('b', metadata, 
+            Column('id', Integer, ForeignKey("a.id"), primary_key=True),
+            Column('data', String(30)),
+            )
+
+        c = Table('c', metadata, 
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            Column('aid', Integer, ForeignKey('a.id', use_alter=True, name="foo")),
+            )
+    def test_flush(self):
+        class A(object):pass
+        class B(A):pass
+        class C(object):pass
+
+        mapper(A, a, properties={
+            'cs':relation(C, primaryjoin=a.c.cid==c.c.id)
+        })
+
+        mapper(B, b, inherits=A, inherit_condition=b.c.id==a.c.id, properties={
+        })
+        mapper(C, c, properties={
+            'arel':relation(A, primaryjoin=a.c.id==c.c.aid)
+        })
+
+        sess = create_session()
+        bobj = B()
+        sess.save(bobj)
+        cobj = C()
+        sess.save(cobj)
+        sess.flush()
         
+
+class BiDirectionalManyToOneTest(ORMTest):
+    def define_tables(self, metadata):
+        global t1, t2, t3, t4
+        t1 = Table('t1', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            Column('t2id', Integer, ForeignKey('t2.id'))
+            )
+        t2 = Table('t2', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            Column('t1id', Integer, ForeignKey('t1.id', use_alter=True, name="foo_fk"))
+            )
+        t3 = Table('t3', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            Column('t1id', Integer, ForeignKey('t1.id'), nullable=False),
+            Column('t2id', Integer, ForeignKey('t2.id'), nullable=False),
+            )
+            
+    def test_reflush(self):
+        class T1(object):pass
+        class T2(object):pass
+        class T3(object):pass
+        
+        mapper(T1, t1, properties={
+            't2':relation(T2, primaryjoin=t1.c.t2id==t2.c.id)
+        })
+        mapper(T2, t2, properties={
+            't1':relation(T1, primaryjoin=t2.c.t1id==t1.c.id)
+        })
+        mapper(T3, t3, properties={
+            't1':relation(T1),
+            't2':relation(T2)
+        })
+        
+        o1 = T1()
+        o1.t2 = T2()
+        sess = create_session()
+        sess.save(o1)
+        sess.flush()
+        
+        # the bug here is that the dependency sort comes up with T1/T2 in a cycle, but there
+        # are no T1/T2 objects to be saved.  therefore no "cyclical subtree" gets generated,
+        # and one or the other of T1/T2 gets lost, and processors on T3 dont fire off.
+        # the test will then fail because the FK's on T3 are not nullable.
+        o3 = T3()
+        o3.t1 = o1
+        o3.t2 = o1.t2
+        sess.save(o3)
+        sess.flush()
+        
+
+    def test_reflush_2(self):
+        """a variant on test_reflush()"""
+        class T1(object):pass
+        class T2(object):pass
+        class T3(object):pass
+
+        mapper(T1, t1, properties={
+            't2':relation(T2, primaryjoin=t1.c.t2id==t2.c.id)
+        })
+        mapper(T2, t2, properties={
+            't1':relation(T1, primaryjoin=t2.c.t1id==t1.c.id)
+        })
+        mapper(T3, t3, properties={
+            't1':relation(T1),
+            't2':relation(T2)
+        })
+
+        o1 = T1()
+        o1.t2 = T2()
+        sess = create_session()
+        sess.save(o1)
+        sess.flush()
+
+        # in this case, T1, T2, and T3 tasks will all be in the cyclical
+        # tree normally.  the dependency processors for T3 are part of the
+        # 'extradeps' collection so they all get assembled into the tree
+        # as well.
+        o1a = T1()
+        o2a = T2()
+        sess.save(o1a)
+        sess.save(o2a)
+        o3b = T3()
+        o3b.t1 = o1a
+        o3b.t2 = o2a
+        sess.save(o3b)
+        
+        o3 = T3()
+        o3.t1 = o1
+        o3.t2 = o1.t2
+        sess.save(o3)
+        sess.flush()
+            
 class BiDirectionalOneToManyTest(AssertMixin):
     """tests two mappers with a one-to-many relation to each other."""
     def setUpAll(self):
         global t1, t2, metadata
-        metadata = BoundMetaData(testbase.db)
+        metadata = MetaData(testbase.db)
         t1 = Table('t1', metadata, 
             Column('c1', Integer, Sequence('t1c1_id_seq', optional=True), primary_key=True),
             Column('c2', Integer, ForeignKey('t2.c1'))
@@ -225,7 +416,7 @@ class BiDirectionalOneToManyTest2(AssertMixin):
     """tests two mappers with a one-to-many relation to each other, with a second one-to-many on one of the mappers"""
     def setUpAll(self):
         global t1, t2, t3, metadata
-        metadata = BoundMetaData(testbase.db)
+        metadata = MetaData(testbase.db)
         t1 = Table('t1', metadata, 
             Column('c1', Integer, Sequence('t1c1_id_seq', optional=True), primary_key=True),
             Column('c2', Integer, ForeignKey('t2.c1')),
@@ -287,7 +478,7 @@ class OneToManyManyToOneTest(AssertMixin):
     raise an exception when dependencies are sorted."""
     def setUpAll(self):
         global metadata
-        metadata = BoundMetaData(testbase.db)
+        metadata = MetaData(testbase.db)
         global person    
         global ball
         ball = Table('ball', metadata,
@@ -320,13 +511,11 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id),
          )
         )
 
-        print str(Person.mapper.props['balls'].primaryjoin)
-        
         b = Ball()
         p = Person()
         p.balls.append(b)
@@ -347,8 +536,8 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id, post_update=False, private=True),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id, post_update=True),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id, post_update=False, private=True),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id, post_update=True),
          )
         )
 
@@ -451,8 +640,8 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id, private=True, post_update=True),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id, private=True, post_update=True, backref='person'),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id),
          )
         )
 
@@ -578,9 +767,10 @@ class OneToManyManyToOneTest(AssertMixin):
         ])
 
 class SelfReferentialPostUpdateTest(AssertMixin):
+    """test using post_update on a single self-referential mapper"""
     def setUpAll(self):
         global metadata, node_table
-        metadata = BoundMetaData(testbase.db)
+        metadata = MetaData(testbase.db)
         node_table = Table('node', metadata,
             Column('id', Integer, Sequence('nodeid_id_seq', optional=True), primary_key=True),
             Column('path', String(50), nullable=False),
@@ -606,19 +796,19 @@ class SelfReferentialPostUpdateTest(AssertMixin):
                 primaryjoin=node_table.c.id==node_table.c.parent_id,
                 lazy=True,
                 cascade="all",
-                backref=backref("parent", primaryjoin=node_table.c.parent_id==node_table.c.id, foreignkey=node_table.c.id)
+                backref=backref("parent", primaryjoin=node_table.c.parent_id==node_table.c.id, remote_side=node_table.c.id)
             ),
             'prev_sibling': relation(
                 Node,
                 primaryjoin=node_table.c.prev_sibling_id==node_table.c.id,
-                foreignkey=node_table.c.id,
+                remote_side=node_table.c.id,
                 lazy=True,
                 uselist=False
             ),
             'next_sibling': relation(
                 Node,
                 primaryjoin=node_table.c.next_sibling_id==node_table.c.id,
-                foreignkey=node_table.c.id,
+                remote_side=node_table.c.id,
                 lazy=True,
                 uselist=False,
                 post_update=True
@@ -670,11 +860,56 @@ class SelfReferentialPostUpdateTest(AssertMixin):
                 "UPDATE node SET next_sibling_id=:next_sibling_id WHERE node.id = :node_id",
                 lambda ctx:{'next_sibling_id':stories.id, 'node_id':about.id}
             ),
+            (
+                "UPDATE node SET next_sibling_id=:next_sibling_id WHERE node.id = :node_id",
+                lambda ctx:{'next_sibling_id':None, 'node_id':cats.id}
+            ),
             (
                 "DELETE FROM node WHERE node.id = :id",
                 lambda ctx:[{'id':cats.id}]
             ),
         ])
+
+class SelfReferentialPostUpdateTest2(AssertMixin):
+    def setUpAll(self):
+        global metadata, a_table
+        metadata = MetaData(testbase.db)
+        a_table = Table("a", metadata,
+                Column("id", Integer(), primary_key=True),
+                Column("fui", String()),
+                Column("b", Integer(), ForeignKey("a.id")),
+            )
+        a_table.create()
+    def tearDownAll(self):
+        a_table.drop()
+    def testbasic(self):
+        """test that post_update remembers to be involved in update operations as well, 
+        since it replaces the normal dependency processing completely [ticket:413]"""
+        class a(object): 
+            def __init__(self, fui):
+                self.fui = fui
+
+        mapper(a, a_table, properties={
+            'foo': relation(a, remote_side=[a_table.c.id], post_update=True),
+        })
+
+        session = create_session()
+
+        f1 = a("f1")
+        session.save(f1)
+        session.flush()
+
+        f2 = a("f2")
+        f2.foo = f1
+        # at this point f1 is already inserted.  but we need post_update
+        # to fire off anyway
+        session.save(f2)
+        session.flush()
+        
+        session.clear()
+        f1 = session.query(a).get(f1.id)
+        f2 = session.query(a).get(f2.id)
+        assert f2.foo is f1
         
 if __name__ == "__main__":
     testbase.main()        
