@@ -14,15 +14,14 @@ class User( Principal ):
 class Group( Principal ):
     pass
 
-class InheritTest(testbase.AssertMixin):
+class InheritTest(testbase.ORMTest):
     """deals with inheritance and many-to-many relationships"""
-    def setUpAll(self):
+    def define_tables(self, metadata):
         global principals
         global users
         global groups
         global user_group_map
-        global metadata
-        metadata = BoundMetaData(testbase.db)
+
         principals = Table(
             'principals',
             metadata,
@@ -57,14 +56,6 @@ class InheritTest(testbase.AssertMixin):
 
             )
 
-        metadata.create_all()
-        
-    def tearDownAll(self):
-        metadata.drop_all()
-
-    def setUp(self):
-        clear_mappers()
-        
     def testbasic(self):
         mapper( Principal, principals )
         mapper( 
@@ -87,11 +78,10 @@ class InheritTest(testbase.AssertMixin):
         sess.flush()
         # TODO: put an assertion
         
-class InheritTest2(testbase.AssertMixin):
+class InheritTest2(testbase.ORMTest):
     """deals with inheritance and many-to-many relationships"""
-    def setUpAll(self):
-        global foo, bar, foo_bar, metadata
-        metadata = BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global foo, bar, foo_bar
         foo = Table('foo', metadata,
             Column('id', Integer, Sequence('foo_id_seq'), primary_key=True),
             Column('data', String(20)),
@@ -105,30 +95,42 @@ class InheritTest2(testbase.AssertMixin):
         foo_bar = Table('foo_bar', metadata,
             Column('foo_id', Integer, ForeignKey('foo.id')),
             Column('bar_id', Integer, ForeignKey('bar.bid')))
-        metadata.create_all()
-    def tearDownAll(self):
-        metadata.drop_all()
 
+    def testget(self):
+        class Foo(object):
+            def __init__(self, data=None):
+                self.data = data
+        class Bar(Foo):pass
+        
+        mapper(Foo, foo)
+        mapper(Bar, bar, inherits=Foo)
+        
+        b = Bar('somedata')
+        sess = create_session()
+        sess.save(b)
+        sess.flush()
+        sess.clear()
+        
+        # test that "bar.bid" does not need to be referenced in a get
+        # (ticket 185)
+        assert sess.query(Bar).get(b.id).id == b.id
+        
     def testbasic(self):
         class Foo(object): 
             def __init__(self, data=None):
                 self.data = data
-            def __str__(self):
-                return "Foo(%s)" % self.data
-            def __repr__(self):
-                return str(self)
 
         mapper(Foo, foo)
         class Bar(Foo):
-            def __str__(self):
-                return "Bar(%s)" % self.data
+            pass
 
         mapper(Bar, bar, inherits=Foo, properties={
             'foos': relation(Foo, secondary=foo_bar, lazy=False)
         })
         
         sess = create_session()
-        b = Bar('barfoo', _sa_session=sess)
+        b = Bar('barfoo')
+        sess.save(b)
         sess.flush()
 
         f1 = Foo('subfoo1')
@@ -147,11 +149,11 @@ class InheritTest2(testbase.AssertMixin):
             {'id':b.id, 'data':'barfoo', 'foos':(Foo, [{'id':f1.id,'data':'subfoo1'}, {'id':f2.id,'data':'subfoo2'}])},
             )
 
-class InheritTest3(testbase.AssertMixin):
+class InheritTest3(testbase.ORMTest):
     """deals with inheritance and many-to-many relationships"""
-    def setUpAll(self):
-        global foo, bar, blub, bar_foo, blub_bar, blub_foo,metadata
-        metadata = BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global foo, bar, blub, bar_foo, blub_bar, blub_foo
+
         # the 'data' columns are to appease SQLite which cant handle a blank INSERT
         foo = Table('foo', metadata,
             Column('id', Integer, Sequence('foo_seq'), primary_key=True),
@@ -176,13 +178,6 @@ class InheritTest3(testbase.AssertMixin):
         blub_foo = Table('blub_foo', metadata,
             Column('blub_id', Integer, ForeignKey('blub.id')),
             Column('foo_id', Integer, ForeignKey('foo.id')))
-        metadata.create_all()
-    def tearDownAll(self):
-        metadata.drop_all()
-
-    def tearDown(self):
-        for table in metadata.table_iterator():
-            table.delete().execute()
             
     def testbasic(self):
         class Foo(object):
@@ -253,11 +248,10 @@ class InheritTest3(testbase.AssertMixin):
         self.echo(x)
         self.assert_(repr(x) == compare)
         
-class InheritTest4(testbase.AssertMixin):
+class InheritTest4(testbase.ORMTest):
     """deals with inheritance and one-to-many relationships"""
-    def setUpAll(self):
-        global foo, bar, blub, metadata
-        metadata = BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global foo, bar, blub
         # the 'data' columns are to appease SQLite which cant handle a blank INSERT
         foo = Table('foo', metadata,
             Column('id', Integer, Sequence('foo_seq'), primary_key=True),
@@ -271,13 +265,6 @@ class InheritTest4(testbase.AssertMixin):
             Column('id', Integer, ForeignKey('bar.id'), primary_key=True),
             Column('foo_id', Integer, ForeignKey('foo.id'), nullable=False),
             Column('data', String(20)))
-        metadata.create_all()
-    def tearDownAll(self):
-        metadata.drop_all()
-
-    def tearDown(self):
-        for table in metadata.table_iterator():
-            table.delete().execute()
 
     def testbasic(self):
         class Foo(object):
@@ -316,12 +303,11 @@ class InheritTest4(testbase.AssertMixin):
         self.assert_(compare == result)
         self.assert_(l[0].parent_foo.data == 'foo #1' and l[1].parent_foo.data == 'foo #1')
 
-class InheritTest5(testbase.AssertMixin):
+class InheritTest5(testbase.ORMTest):
     """testing that construction of inheriting mappers works regardless of when extra properties
     are added to the superclass mapper"""
-    def setUpAll(self):
-        global content_type, content, product, metadata
-        metadata = BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global content_type, content, product
         content_type = Table('content_type', metadata, 
             Column('id', Integer, primary_key=True)
             )
@@ -332,10 +318,6 @@ class InheritTest5(testbase.AssertMixin):
         product = Table('product', metadata, 
             Column('id', Integer, ForeignKey('content.id'), primary_key=True)
         )
-    def tearDownAll(self):
-        pass
-    def tearDown(self):
-        pass
 
     def testbasic(self):
         class ContentType(object): pass
@@ -366,12 +348,11 @@ class InheritTest5(testbase.AssertMixin):
         p.contenttype = ContentType()
         # TODO: assertion ??
         
-class InheritTest6(testbase.AssertMixin):
+class InheritTest6(testbase.ORMTest):
     """tests eager load/lazy load of child items off inheritance mappers, tests that
     LazyLoader constructs the right query condition."""
-    def setUpAll(self):
-        global foo, bar, bar_foo, metadata
-        metadata=BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global foo, bar, bar_foo
         foo = Table('foo', metadata, Column('id', Integer, Sequence('foo_seq'), primary_key=True),
         Column('data', String(30)))
         bar = Table('bar', metadata, Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
@@ -381,10 +362,7 @@ class InheritTest6(testbase.AssertMixin):
         Column('bar_id', Integer, ForeignKey('bar.id')),
         Column('foo_id', Integer, ForeignKey('foo.id'))
         )
-        metadata.create_all()
-    def tearDownAll(self):
-        metadata.drop_all()
-        
+
     def testbasic(self):
         class Foo(object): pass
         class Bar(Foo): pass
@@ -413,11 +391,10 @@ class InheritTest6(testbase.AssertMixin):
         self.assert_(len(q.selectfirst().eager) == 1)
 
 
-class InheritTest7(testbase.AssertMixin):
+class InheritTest7(testbase.ORMTest):
     """test dependency sorting among inheriting mappers"""
-    def setUpAll(self):
-        global users, roles, user_roles, admins, metadata
-        metadata=BoundMetaData(testbase.db)
+    def define_tables(self, metadata):
+        global users, roles, user_roles, admins
         users = Table('users', metadata,
             Column('id', Integer, primary_key=True),
             Column('email', String(128)),
@@ -438,23 +415,19 @@ class InheritTest7(testbase.AssertMixin):
             Column('id', Integer, primary_key=True),
             Column('user_id', Integer, ForeignKey('users.id'))
         )
-        metadata.create_all()
-    def tearDownAll(self):
-        metadata.drop_all()
-    def tearDown(self):
-        for t in metadata.table_iterator(reverse=True):
-            t.delete().execute()
             
     def testone(self):
         class User(object):pass
-        class Role(object):pass
+        class Role(object):
+            def __init__(self, description):
+                self.description = description
         class Admin(User):pass
         role_mapper = mapper(Role, roles)
         user_mapper = mapper(User, users, properties = {
                 'roles' : relation(Role, secondary=user_roles, lazy=False, private=False) 
             }
         )
-        admin_mapper = mapper(Admin, admins, inherits=user_mapper)
+        admin_mapper = mapper(Admin, admins, inherits=user_mapper, properties={'aid':admins.c.id})
         sess = create_session()
         adminrole = Role('admin')
         sess.save(adminrole)
@@ -490,7 +463,7 @@ class InheritTest7(testbase.AssertMixin):
             }
         )
 
-        admin_mapper = mapper(Admin, admins, inherits=user_mapper) 
+        admin_mapper = mapper(Admin, admins, inherits=user_mapper, properties={'aid':admins.c.id}) 
 
         # create roles
         adminrole = Role('admin')
@@ -508,6 +481,95 @@ class InheritTest7(testbase.AssertMixin):
         a.password = 'sadmin'
         sess.flush()
         assert user_roles.count().scalar() == 1
+
+class InheritTest8(testbase.ORMTest):
+    """test the construction of mapper.primary_key when an inheriting relationship
+    joins on a column other than primary key column."""
+    keep_data = True
+    
+    def define_tables(self, metadata):
+        global person_table, employee_table, Person, Employee
+        
+        person_table = Table("persons", metadata,
+                Column("id", Integer, primary_key=True),
+                Column("name", String(80)),
+                )
+
+        employee_table = Table("employees", metadata,
+                Column("id", Integer, primary_key=True),
+                Column("salary", Integer),
+                Column("person_id", Integer, ForeignKey("persons.id")),
+                )
+
+        class Person(object):
+            def __init__(self, name):
+                self.name = name
+
+        class Employee(Person): pass
+
+        import warnings
+        warnings.filterwarnings("error", r".*On mapper.*distinct primary key")
+    
+    def insert_data(self):
+        person_insert = person_table.insert()
+        person_insert.execute(id=1, name='alice')
+        person_insert.execute(id=2, name='bob')
+
+        employee_insert = employee_table.insert()
+        employee_insert.execute(id=2, salary=250, person_id=1) # alice
+        employee_insert.execute(id=3, salary=200, person_id=2) # bob
+        
+    def test_implicit(self):
+        person_mapper = mapper(Person, person_table)
+        mapper(Employee, employee_table, inherits=person_mapper)
+        try:
+            print class_mapper(Employee).primary_key
+            assert list(class_mapper(Employee).primary_key) == [person_table.c.id, employee_table.c.id]
+            assert False
+        except RuntimeWarning, e:
+            assert str(e) == "On mapper Mapper|Employee|employees, primary key column 'employees.id' is being combined with distinct primary key column 'persons.id' in attribute 'id'.  Use explicit properties to give each column its own mapped attribute name."
+
+    def test_explicit_props(self):
+        person_mapper = mapper(Person, person_table)
+        mapper(Employee, employee_table, inherits=person_mapper, properties={'pid':person_table.c.id, 'eid':employee_table.c.id})
+        self._do_test(True)
+    
+    def test_explicit_composite_pk(self):
+        person_mapper = mapper(Person, person_table)
+        mapper(Employee, employee_table, inherits=person_mapper, primary_key=[person_table.c.id, employee_table.c.id])
+        try:
+            self._do_test(True)
+            assert False
+        except RuntimeWarning, e:
+            assert str(e) == "On mapper Mapper|Employee|employees, primary key column 'employees.id' is being combined with distinct primary key column 'persons.id' in attribute 'id'.  Use explicit properties to give each column its own mapped attribute name."
+
+    def test_explicit_pk(self):
+        person_mapper = mapper(Person, person_table)
+        mapper(Employee, employee_table, inherits=person_mapper, primary_key=[person_table.c.id])
+        self._do_test(False)
+        
+    def _do_test(self, composite):
+        session = create_session()
+        query = session.query(Employee)
+
+        if composite:
+            try:
+                query.get(1)
+                assert False
+            except exceptions.InvalidRequestError, e:
+                assert str(e) == "Could not find enough values to formulate primary key for query.get(); primary key columns are 'persons.id', 'employees.id'"
+            alice1 = query.get([1,2])
+            bob = query.get([2,3])
+            alice2 = query.get([1,2])
+        else:
+            alice1 = query.get(1)
+            bob = query.get(2)
+            alice2 = query.get(1)
+            
+            assert alice1.name == alice2.name == 'alice'
+            assert bob.name == 'bob'
+        
+
         
 if __name__ == "__main__":    
     testbase.main()
