@@ -24,13 +24,13 @@ class DefaultTest(PersistTest):
         # also correct for "current_date" compatible as column default, value differences
         currenttime = func.current_date(type=Date, engine=db);
         if is_oracle:
-            ts = db.func.trunc(func.sysdate(), column("'DAY'")).scalar()
+            ts = db.func.trunc(func.sysdate(), literal_column("'DAY'")).scalar()
             f = select([func.count(1) + 5], engine=db).scalar()
             f2 = select([func.count(1) + 14], engine=db).scalar()
             # TODO: engine propigation across nested functions not working
-            currenttime = func.trunc(currenttime, column("'DAY'"), engine=db)
+            currenttime = func.trunc(currenttime, literal_column("'DAY'"), engine=db)
             def1 = currenttime
-            def2 = func.trunc(text("sysdate"), column("'DAY'"))
+            def2 = func.trunc(text("sysdate"), literal_column("'DAY'"))
             deftype = Date
         elif use_function_defaults:
             f = select([func.count(1) + 5], engine=db).scalar()
@@ -127,7 +127,7 @@ class DefaultTest(PersistTest):
 class AutoIncrementTest(PersistTest):
     @testbase.supported('postgres', 'mysql')
     def testnonautoincrement(self):
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         nonai_table = Table("aitest", meta, 
             Column('id', Integer, autoincrement=False, primary_key=True),
             Column('data', String(20)))
@@ -147,7 +147,7 @@ class AutoIncrementTest(PersistTest):
             nonai_table.drop()    
 
     def testwithautoincrement(self):
-        meta = BoundMetaData(testbase.db)
+        meta = MetaData(testbase.db)
         table = Table("aitest", meta, 
             Column('id', Integer, primary_key=True),
             Column('data', String(20)))
@@ -158,18 +158,61 @@ class AutoIncrementTest(PersistTest):
         finally:
             table.drop()    
 
+    def testfetchid(self):
+        meta = MetaData(testbase.db)
+        table = Table("aitest", meta, 
+            Column('id', Integer, primary_key=True),
+            Column('data', String(20)))
+        table.create()
+
+        try:
+            # simulate working on a table that doesn't already exist
+            meta2 = MetaData(testbase.db)
+            table2 = Table("aitest", meta2,
+                Column('id', Integer, primary_key=True),
+                Column('data', String(20)))
+            class AiTest(object):
+                pass
+            mapper(AiTest, table2)
+        
+            s = create_session()
+            u = AiTest()
+            s.save(u)
+            s.flush()
+            assert u.id is not None
+            s.clear()
+        finally:
+            table.drop()
+        
+
 class SequenceTest(PersistTest):
     @testbase.supported('postgres', 'oracle')
     def setUpAll(self):
-        global cartitems
-        cartitems = Table("cartitems", db, 
+        global cartitems, sometable, metadata
+        metadata = MetaData(testbase.db)
+        cartitems = Table("cartitems", metadata, 
             Column("cart_id", Integer, Sequence('cart_id_seq'), primary_key=True),
             Column("description", String(40)),
             Column("createdate", DateTime())
         )
+        sometable = Table( 'Manager', metadata,
+               Column( 'obj_id', Integer, Sequence('obj_id_seq'), ),
+               Column( 'name', type= String, ),
+               Column( 'id', Integer, primary_key= True, ),
+           )
         
-        cartitems.create()
+        metadata.create_all()
     
+    @testbase.supported('postgres', 'oracle')
+    def testseqnonpk(self):
+        """test sequences fire off as defaults on non-pk columns"""
+        sometable.insert().execute(name="somename")
+        sometable.insert().execute(name="someother")
+        assert sometable.select().execute().fetchall() == [
+            (1, "somename", 1),
+            (2, "someother", 2),
+        ]
+        
     @testbase.supported('postgres', 'oracle')
     def testsequence(self):
         cartitems.insert().execute(description='hi')
@@ -181,13 +224,23 @@ class SequenceTest(PersistTest):
    
     @testbase.supported('postgres', 'oracle')
     def teststandalone(self):
-        s = Sequence("my_sequence", metadata=testbase.db)
+        s = Sequence("my_sequence", metadata=MetaData(testbase.db))
         s.create()
         try:
             x = s.execute()
             self.assert_(x == 1)
         finally:
             s.drop()
+
+    @testbase.supported('postgres', 'oracle')
+    def teststandalone_explicit(self):
+        s = Sequence("my_sequence")
+        s.create(bind=testbase.db)
+        try:
+            x = s.execute(testbase.db)
+            self.assert_(x == 1)
+        finally:
+            s.drop(testbase.db)
     
     @testbase.supported('postgres', 'oracle')
     def teststandalone2(self):
@@ -196,7 +249,7 @@ class SequenceTest(PersistTest):
         
     @testbase.supported('postgres', 'oracle')
     def tearDownAll(self): 
-        cartitems.drop()
+        metadata.drop_all()
 
 if __name__ == "__main__":
     testbase.main()
